#!/bin/bash

# script  will set the approbiate settings for Client/AP Mode
# to show wget outputs delete parameter  -q    or add parameter -d for even more debug-infos

LOGFILE=/var/log/lcdd_scripts
AP_SSID_FILE=/etc/robotino/AP_SSID

myecho()
{
	echo "$1"
	echo "$1" >> $LOGFILE
}

error_exit()
{
	myecho "Error: $1"
	exit 1
}

AP_SSID=""
if [ -e "$AP_SSID_FILE" ] ; then
	AP_SSID=`cat $AP_SSID_FILE`
fi
if [ -z "$AP_SSID" ] ; then
	AP_SSID=Robotino000.000
fi

ALLNET_IP=172.26.101.1
WGETPARAM="--auth-no-challenge  --timeout=8 --tries=2 --http-user=admin --http-passwd=apap6 --wait=2 -q"   #-q  #-d



myecho "Searching 2x8 seconds for Allnet-AccessPoint."
wget http://${ALLNET_IP}/status.asp $WGETPARAM   -O- | grep "wlanMode\[0] =0;" || error_exit "No Allnet found or Allnet not in AP mode"

myecho "Setup SSID $AP_SSID in AP-mode..."
wget http://$ALLNET_IP/goform/formWlanSetup     --post-data="ssid0=$AP_SSID&wlan-url=%2Fwlbasic.asp&save=Apply+Changes&basicrates0=15&operrates0=4095"  $WGETPARAM -O-  || error_exit "Setup new SSID in AP mode"

myecho "Setup DHCP=ON ..."
wget http://$ALLNET_IP/goform/formTcpipSetup    --post-data="dhcp=2&save=Apply+Changes&submit-url=%2Ftcpip.asp"  $WGETPARAM  -O- || error_exit "Setup DHCP=ON"

myecho "wait 20 sec for AP-restart..."
for i in {1..20}
do
	myecho "."
	sleep 1
done

myecho "Check if new AP-SSID has been written"
wget http://$ALLNET_IP/status.asp $WGETPARAM -O- | grep "^ssid_drv\[0\] ='$AP_SSID';" || error_exit "Failed writing new SSID"
myecho "Success"

exit 0

