#!/bin/bash

# script  will set the approbiate settings for Client/AP Mode
# to show wget outputs delete parameter  -q    or add parameter -d for even more debug-infos

LOGFILE=/var/log/lcdd_scripts
AP_SSID_FILE=/etc/robotino/AP_SSID

CL_SSID=`cat /etc/robotino/CLIENT_SSID`
#wenn cat fehlschlug oder wenn CL_SSID leer ist
if [ 1 -eq $? -o -z "$CL_SSID" ] ; then
	CL_SSID=RobotinoAPX.1
fi

ALLNET_IP=172.26.101.1
WGETPARAM="--auth-no-challenge  --timeout=8 --tries=2 --http-user=admin --http-passwd=apap6 --wait=2 -q"   #-q  #-d

myecho()
{
	echo "$1"
	echo "$1" >> $LOGFILE
}

error_exit()
{
	myecho "Error: $1"
	exit 1
}

AP_SSID=""
if [ -e "$AP_SSID_FILE" ] ; then
	AP_SSID=`cat $AP_SSID_FILE`
fi
if [ -z "$AP_SSID" ] ; then
	myecho "Reading SSID and store it in $AP_SSID_FILE"
	AP_SSID=`wget http://${ALLNET_IP}/status.asp $WGETPARAM   -O- | grep "^ssid_drv\[0\] =\(.*\);" | sed -n "s/ssid_drv\[0] =//;s/\;//;s/'//g;p"`
	myecho "Read SSID: $AP_SSID"
	echo $AP_SSID > $AP_SSID_FILE
fi

myecho "Searching 2x8 seconds for Allnet-AccessPoint."
wget http://${ALLNET_IP}/status.asp $WGETPARAM   -O- | grep "wlanMode\[0] =1;" || error_exit "No Allnet found or Allnet not in CL mode"

myecho "Setup SSID $CL_SSID in CL-mode..."
wget http://$ALLNET_IP/goform/formWlanSetup     --post-data="ssid0=$CL_SSID&wlan-url=%2Fwlbasic.asp&save=Apply+Changes&basicrates0=15&operrates0=4095"  $WGETPARAM -O-  || error_exit "Setup new SSID in CL mode"

myecho "Setup DHCP=OFF ..."
wget http://$ALLNET_IP/goform/formTcpipSetup    --post-data="dhcp=0&save=Apply+Changes&submit-url=%2Ftcpip.asp"  $WGETPARAM  -O- || error_exit "Setup DHCP=OFF"

myecho "wait 20 sec for AP-restart..."
for i in {1..20}
do
	myecho "."
	sleep 1
done

myecho "Check if new CL-SSID has been written"
wget http://$ALLNET_IP/status.asp $WGETPARAM -O- | grep "^ssid_drv\[0\] ='$CL_SSID';" || error_exit "Failed writing new SSID"
myecho "Success"

exit 0

