//  Copyright (C) 2004-2009, Robotics Equipment Corporation GmbH

#include "CameraView.h"

#include "rec/core_lt/image/conv.h"
#include <iostream>

CameraView::CameraView():
_tracker(QString(ARTOOLKIT_DATA_PATH)+ "/camera_para.dat", QSize(320,240),NULL)
{
	rec::robotino::com::Camera::setResolution( 320, 240 );
	rec::robotino::com::Camera::setStreaming( true );
	setMinimumSize( 320, 240 );
	_tracker.registerMarker(QString(ARTOOLKIT_DATA_PATH)+ "/patterns/patt.hiro",0);
}

void CameraView::paintEvent( QPaintEvent* e )
{
	QPainter painter( this );

	painter.drawImage( rect(), _image );
}


std::vector<float> vectorMatrixMult(std::vector<std::vector<float> > mat, std::vector<float> vec)
{
	assert(mat[0].size() == vec.size());
	std::vector<float> ret;
	ret.reserve(4);

	for( int row = 0; row < 4; ++row )
	{
		ret.push_back( mat[row][0] * vec[0] + mat[row][1] * vec[1] + mat[row][2] * vec[2] + mat[row][3] * vec[3]);
	}

//	for( int row = 0; row < 4; ++row )
//	{
//		vec[row] /= vec[3];
//	}

	return ret;
}


void CameraView::imageReceivedEvent( const unsigned char* data,
									unsigned int dataSize,
									unsigned int width,
									unsigned int height,
									unsigned int numChannels,
									unsigned int bytesPerChannel,
									unsigned int step )
{
	QImage newImage( data, width, height, QImage::Format_RGB888 );
	_image = newImage.copy();
	_tracker.setImage(_image);
	if( !_image.isNull() )
	{
		TIdToMarkerMap detectedMarkers = _tracker.trackMarkers();

		if(detectedMarkers.contains(0))
		{
			QPainter p(&_image);
			p.setPen(qRgb(255,0,0));
			p.drawRect(QRect(detectedMarkers[0].pos.toPoint()-QPoint(3,3),QSize(6, 6) ));
			std::vector<float> desiredSpotInMarkerSpace;
			desiredSpotInMarkerSpace.push_back(0);desiredSpotInMarkerSpace.push_back(0);desiredSpotInMarkerSpace.push_back(500);
			desiredSpotInMarkerSpace.push_back(1);
			std::vector<float> desiredSpotInRobotSpace(vectorMatrixMult(detectedMarkers[0].trans, desiredSpotInMarkerSpace));

			QPointF target(desiredSpotInRobotSpace[2], desiredSpotInRobotSpace[0]);
			QPointF direction = target;
			float omega = detectedMarkers[0].pos.x() - (width/2.0f);
			_omniDrive.setVelocity(direction.x(), -direction.y(), -omega);
		}
		else
		{
			_omniDrive.setVelocity(0,0,0);
		}
	}
	update();
}


