/*
 * MarkerTracker.cpp
 *
 *  Created on: Apr 28, 2010
 *      Author: andreast
 */

#include "MarkerTracker.h"
#include <AR/config.h>
#include <QtGui/QPainter>
#include <QSettings>
#include <QDebug>


/*
// Marker detection.
static int			gARTThreshhold = 100;
static long			gCallCountMarkerDetect = 0;

// Transformation matrix retrieval.
static double		gPatt_width     = 80.0;	// Per-marker, but we are using only 1 marker.
static double		gPatt_centre[2] = {0.0, 0.0}; // Per-marker, but we are using only 1 marker.
static double		gPatt_trans[3][4];		// Per-marker, but we are using only 1 marker.
static int			gPatt_found = FALSE;	// Per-marker, but we are using only 1 marker.
static int			gPatt_id;				// Per-marker, but we are using only 1 marker.

// Drawing.
static ARParam		gARTCparam;

char *cparam_name = "Data/camera_para.dat";
char *vconf = "";
char *patt_name  = "Data/patt.hiro";
*/
int MarkerTracker::artkInit(const QString& cameraParamFileName, QSize imageSize)
{
    ARParam			wparam;
	// Load the camera parameters, resize for the window and init.
    if (arParamLoad(cameraParamFileName.toLatin1(), 1, &wparam) < 0) {
		qDebug() << "Camera parameter loading failed!" << cameraParamFileName;
        return (FALSE);
    }
    arParamChangeSize(&wparam, imageSize.width(), imageSize.height(), &mCameraParam);
    fprintf(stdout, "*** Camera Parameter ***\n");
    arParamDisp(&mCameraParam);

    arInitCparam(&mCameraParam);
    return true;
}



void MarkerTracker::registerMarker(const QString& patternFile, TMarkerId id)
{
	int patt_id = arLoadPatt(patternFile.toLatin1());

	if( patt_id < 0 )
	{
		fprintf(stderr, "setupMarker(): pattern load error !!\n");
		return;
	}
	mPattIdToMarkerId[patt_id] = id;
	mPatternIds.insert(patt_id);
}


MarkerTracker::MarkerTracker(const QString& cameraParamFile, const QSize& imageSize, QObject* parent)
: mHaveNewImage(false), mpTimer(new QTimer()), QThread(parent)
{
	artkInit(cameraParamFile, imageSize);
}

MarkerTracker::~MarkerTracker()
{
	// TODO Auto-generated destructor stub
}


void MarkerTracker::setImage(QImage img)
{
	{QMutexLocker lock (&mImgMutex);
		mImg = img;
	}
}

void MarkerTracker::run()
{
}

TIdToMarkerMap MarkerTracker::trackMarkers()
{
	ARMarkerInfo    *marker_info;					// Pointer to array holding the details of detected markers.
	int             marker_num;						// Count of number of markers detected.

	{QMutexLocker lock (&mImgMutex);
		// Detect the markers in the video frame.
		if (arDetectMarker(mImg.bits(), 100, &marker_info, &marker_num) < 0)
		{
			qDebug("ARTK:arDetectMarker failed");
		}
	}

	typedef std::map<int, std::pair<int, double> > TPattIdToIndexNConfidenceMap;
	TPattIdToIndexNConfidenceMap pattIdToIndexNConfidence;
	for (int j = 0; j < marker_num; j++)
	{
		if (mPatternIds.find(marker_info[j].id) != mPatternIds.end())
		{
			// ok we know the pattern...
			TPattIdToIndexNConfidenceMap::iterator it;
			if ((it = pattIdToIndexNConfidence.find(marker_info[j].id)) != pattIdToIndexNConfidence.end())
			{
				//we had that pattern before, check the confidence:
				if ((*it).second.second < marker_info[j].cf)
				{
					(*it).second = std::pair<int,double>(j, marker_info[j].cf);
				}
			}
			else
			{
				// this is an unprocessed pattern:
				pattIdToIndexNConfidence[marker_info[j].id] = std::pair<int,double>(j, marker_info[j].cf);
			}
		}
	}

	// just a carryover from old code...
	double		gPatt_width     = 80.0;	// Per-marker, but we are using only 1 marker.
	double		gPatt_centre[2] = {0.0, 0.0}; // Per-marker, but we are using only 1 marker.
	double		gPatt_trans[3][4];		// Per-marker, but we are using only 1 marker.

	// fill detectedMarkers:
	TIdToMarkerMap detectedMarkers;
	for(TPattIdToIndexNConfidenceMap::const_iterator it = pattIdToIndexNConfidence.begin(); it != pattIdToIndexNConfidence.end(); ++it)
	{
		arGetTransMat(&marker_info[it->second.first], gPatt_centre, gPatt_width, gPatt_trans);
		MarkerInfo markerInfo = marker_info[it->second.first];
		for(int row = 0; row < 3; ++row)
		{
			markerInfo.trans.push_back(std::vector<float>());
			markerInfo.trans[row].push_back(gPatt_trans[row][0]);
			markerInfo.trans[row].push_back(gPatt_trans[row][1]);
			markerInfo.trans[row].push_back(gPatt_trans[row][2]);
			markerInfo.trans[row].push_back(gPatt_trans[row][3]);
		}
		// add 4th row (homogenous coordinates!):
		markerInfo.trans.push_back(std::vector<float>());
		markerInfo.trans[3].push_back(0);
		markerInfo.trans[3].push_back(0);
		markerInfo.trans[3].push_back(0);
		markerInfo.trans[3].push_back(1);

		detectedMarkers[mPattIdToMarkerId[it->first]] = markerInfo;

		if( mVisualizing )
		{
			int k = it->second.first;
			QPolygon markerPoly;
			markerPoly << QPoint(marker_info[k].vertex[0][0], marker_info[k].vertex[0][1])
						<< QPoint(marker_info[k].vertex[1][0], marker_info[k].vertex[1][1])
						<< QPoint(marker_info[k].vertex[2][0], marker_info[k].vertex[2][1])
						<< QPoint(marker_info[k].vertex[3][0], marker_info[k].vertex[3][1]);
			{
				QPainter p(&mImg);
				p.setPen(Qt::green);
				p.drawPolygon(markerPoly);
				p.setPen(Qt::red);
				p.drawEllipse(QRect(markerInfo.pos.x(), markerInfo.pos.y(), 2,2));
			}
		}
	}
	if( mVisualizing)
	{
		Q_EMIT signalNewVisImage(mImg);
	}

	return detectedMarkers;
}
