/*
 * MarkerTracker.h
 *
 *  Created on: Apr 28, 2010
 *      Author: andreast
 */

#ifndef MARKERTRACKER_H_
#define MARKERTRACKER_H_

#include <QObject>
#include <QImage>
#include <QMutex>
#include <set>
#include <map>
#include <QMatrix>
#include <QImage>
#include <QString>
#include <QMap>
#include <AR/param.h>
#include <AR/ar.h>
#include <QTimer>
#include <QThread>
#include <QSharedPointer>
#include "config_data.h"


typedef int TMarkerId;

//typedef ARMarkerInfo MarkerInfo;

struct MarkerInfo : public ARMarkerInfo{
	MarkerInfo()
	: area(0), id(0), dir(0), cf(0), pos(0,0), vertex(4,QPoint(0,0))
	{
	};

	MarkerInfo(const ARMarkerInfo& arinfo)
	: area(arinfo.area), id(arinfo.id), dir(arinfo.dir), cf(arinfo.cf),
	  pos(arinfo.pos[0], arinfo.pos[1])
	{
		vertex.reserve(4);
		vertex.push_back(QPoint(arinfo.vertex[0][0],arinfo.vertex[0][1]));
		vertex.push_back(QPoint(arinfo.vertex[1][0],arinfo.vertex[1][1]));
		vertex.push_back(QPoint(arinfo.vertex[2][0],arinfo.vertex[2][1]));
		vertex.push_back(QPoint(arinfo.vertex[3][0],arinfo.vertex[3][1]));
	};

	int     area;
	int     id;
	int     dir;
	double  cf;
	QPointF pos;
	double  line[4][3];
	std::vector<QPoint>  vertex;
	std::vector<std::vector<float> > trans;
};


typedef QMap<TMarkerId, MarkerInfo> TIdToMarkerMap;
typedef TIdToMarkerMap* TSPIdToMarkerMap;

class MarkerTracker: public QThread
{
	Q_OBJECT

public:

public:
	MarkerTracker(const QString& cameraParamFile, const QSize& imageSize, QObject* parent=0);
	virtual ~MarkerTracker();

	void setVisualizing(bool doVis) {mVisualizing = doVis;};
	bool visualizing() {return mVisualizing;};
	void registerMarker(const QString& patternFile, TMarkerId id);

	TIdToMarkerMap trackMarkers();


Q_SIGNALS:
	void signalMarkerDetected(TSPIdToMarkerMap detectedMarkers);
	void signalNewVisImage( QImage visImg );

public Q_SLOTS:
	void setImage(QImage img);


private:
	MarkerTracker(const MarkerTracker& other){}; // non copyable!
	QImage mImg;
	std::map<int,TMarkerId>  mPattIdToMarkerId;
	std::set<int> mPatternIds;
	QMutex mImgMutex;
	bool mVisualizing;
	bool mHaveNewImage;
	ARParam mCameraParam;
	int artkInit(const QString& cameraParamFileName, QSize imageSize);
	QTimer* mpTimer;

	void run();
};

#endif /* MARKERTRACKER_H_ */
