//  Copyright (C) 2004-2008, Robotics Equipment Corporation GmbH

#define _USE_MATH_DEFINES
#include <cmath>
#include <iostream>

#include "rec/robotino/com/all.h"
#include "rec/core_lt/utils.h"
#include "rec/core_lt/Timer.h"
#include "rec/core_lt/StringList.h"

using namespace rec::robotino::com;

class MyCom : public Com
{
public:
	MyCom()
	{
	}

	void errorEvent( Error error, const char* errorString )
	{
		std::cerr << "Error: " << errorString << std::endl;
	}

	void connectedEvent()
	{
		std::cout << "Connected." << std::endl;
	}

	void connectionClosedEvent()
	{
		std::cout << "Connection closed." << std::endl;
	}

	void modeChangedEvent( bool isPassiveMode )
	{
		if( isPassiveMode )
		{
			std::cout << "Connected int passive mode." << std::endl;
		}
	}
};

class MyBHA : public CompactBHA
{
public:
	MyBHA()
	{
	}

	virtual void readingsCallback( const std::vector<float>& pressures )
	{
		for( unsigned int i=0; i<pressures.size(); ++i )
		{
			std::cout << "B" << i+1 << ": " << pressures[i] << " bar" << std::endl;
		}
		std::cout << std::endl;
	}

	virtual void error( const std::string& message )
	{
		std::cerr << message << std::endl;
	}

	virtual void log( const std::string& message )
	{
		std::cerr << message << std::endl;
	}

	virtual void connected()
	{
		std::cerr << "Connected" << std::endl;
	}

	virtual void disconnected()
	{
		std::cerr << "Disconnected" << std::endl;
	}
};


MyCom com;
MyBHA bha;

void init( const std::string& hostname )
{
	// Initialize the actors

	// Connect
	std::cout << "Connecting..." << std::endl;
	com.setAddress( hostname.c_str() );

	com.connect();
}

void drive()
{
	std::vector<float> pressures( 8 );
	memset( &pressures[0], 0, pressures.size() * sizeof( float ) );

	while( com.isConnected() )
	{
		bha.setPressures( pressures );
		rec::core_lt::msleep( 200 );
	}
}

void destroy()
{
	com.disconnect();
}

int main( int argc, char **argv )
{
	std::string hostname = "192.168.1.212";
	if( argc > 1 )
	{
		hostname = argv[1];
	}

	try
	{
		init( hostname );
		drive();
		destroy();
	}
	catch( const rec::robotino::com::ComException& e )
	{
		std::cerr << "Com Error: " << e.what() << std::endl;
	}
	catch( const std::exception& e )
	{
		std::cerr << "Error: " << e.what() << std::endl;
	}
	catch( ... )
	{
		std::cerr << "Unknow Error" << std::endl;
	}

	//std::cout << "Press any key to exit..." << std::endl;
	//rec::core_lt::waitForKey();
}
