//  Copyright (C) 2004-2008, Robotics Equipment Corporation GmbH

#define _USE_MATH_DEFINES
#include <cmath>
#include <iostream>

#include "rec/robotino/com/all.h"
#include "rec/core_lt/utils.h"
#include "rec/core_lt/Timer.h"

using namespace rec::robotino::com;

class MyCom : public Com
{
public:
	MyCom()
	{
	}

	void errorEvent( Com::Error error, const char* errorString )
	{
		std::cerr << "Error: " << errorString << std::endl;
	}

	void connectedEvent()
	{
		std::cout << "Connected." << std::endl;
	}

	void connectionClosedEvent()
	{
		std::cout << "Connection closed." << std::endl;
	}
};

MyCom com;
Gripper gripper;

void init( const std::string& hostname )
{
	// Initialize the actors

	// Connect
	std::cout << "Connecting..." << std::endl;
	com.setAddress( hostname.c_str() );

	com.connect();

	std::cout << std::endl << "Connected" << std::endl;
}

void drive()
{
	bool asdf = true;
	while( com.isConnected() )
	{
		//std::cout << "isOpenend" << (int) gripper.isOpened() << std::endl;
		if( gripper.isOpened() && asdf )
		{
			gripper.close();
			asdf = false;
		}

		if( gripper.isClosed() && asdf )
		{
			gripper.open();
			asdf = false;
		}

		com.waitForUpdate();
	}
}

void destroy()
{
	com.disconnect();
}

int main( int argc, char **argv )
{
	std::string hostname = "172.26.1.1";
	if( argc > 1 )
	{
		hostname = argv[1];
	}

	try
	{
		init( hostname );
		drive();
		destroy();
	}
	catch( const rec::robotino::com::ComException& e )
	{
		std::cerr << "Com Error: " << e.what() << std::endl;
	}
	catch( const std::exception& e )
	{
		std::cerr << "Error: " << e.what() << std::endl;
	}
	catch( ... )
	{
		std::cerr << "Unknow Error" << std::endl;
	}

	//std::cout << "Press any key to exit..." << std::endl;
	//rec::core_lt::waitForKey();
}
