//  Copyright (C) 2004-2008, Robotics Equipment Corporation GmbH

#define _USE_MATH_DEFINES
#include <cmath>
#include <iostream>

#include "rec/robotino/com/all.h"
#include "rec/core_lt/utils.h"
#include "rec/core_lt/Timer.h"

#define VELOCITY 200.0f //mm/s
#define ANGULARVELOCITY 30.0f //deg/s

using namespace rec::robotino::com;

class MyCom : public Com
{
public:
	MyCom()
	{
	}

	void errorEvent( Error error, const char* errorString )
	{
		std::cerr << "Error: " << errorString << std::endl;
	}

	void connectedEvent()
	{
		std::cout << "Connected." << std::endl;
	}

	void connectionClosedEvent()
	{
		std::cout << "Connection closed." << std::endl;
	}
};

MyCom com;
Motor motor1;
Motor motor2;
Motor motor3;
OmniDrive omniDrive;
Bumper bumper;
AnalogInput analogInput0;

void init( const std::string& hostname )
{
	// Initialize the actors
	motor1.setComId( com.id() );
	motor1.setMotorNumber( 0 );

	motor2.setComId( com.id() );
	motor2.setMotorNumber( 1 );

	motor3.setComId( com.id() );
	motor3.setMotorNumber( 2 );

	omniDrive.setComId( com.id() );

	bumper.setComId( com.id() );

	analogInput0.setComId( com.id() );
	analogInput0.setInputNumber( 0 );

	// Connect
	std::cout << "Connecting..." << std::endl;
	com.setAddress( hostname.c_str() );

	com.connect();

	std::cout << std::endl << "Connected" << std::endl;
}

void drive()
{
	float omega = 0.0;
	float domega = 0.0;
	float velocity = 0;
	float dvelocity = 0;
	float imax = 10.0;
	float imin = 1.0;

	while( com.isConnected() && false == bumper.value() )
	{
		float inductive = 2.0f * ( analogInput0.value() - imin ) / ( imax - imin ) - 1;

		domega = ANGULARVELOCITY * inductive - omega ;
		omega += 0.5 * domega;

		if( -0.5f < inductive && inductive < 0.5f )
		{
			velocity = VELOCITY;
		}
		else
		{
			dvelocity = - velocity;
			velocity += 0.002f * dvelocity;
		}

		omniDrive.setVelocity( velocity, 0, omega );

		com.waitForUpdate();
	}
}

void destroy()
{
	com.disconnect();
}

int main( int argc, char **argv )
{
	std::string hostname = "172.26.1.1";
	if( argc > 1 )
	{
		hostname = argv[1];
	}

	try
	{
		init( hostname );
		drive();
		destroy();
	}
	catch( const rec::robotino::com::ComException& e )
	{
		std::cerr << "Com Error: " << e.what() << std::endl;
	}
	catch( const std::exception& e )
	{
		std::cerr << "Error: " << e.what() << std::endl;
	}
	catch( ... )
	{
		std::cerr << "Unknow Error" << std::endl;
	}

	//std::cout << "Press any key to exit..." << std::endl;
	//rec::core_lt::waitForKey();
}
