//  Copyright (C) 2004-2008, Robotics Equipment Corporation GmbH

#define _USE_MATH_DEFINES
#include <iostream>
#include <map>

#include "rec/robotino/com/all.h"
#include "rec/core_lt/utils.h"
#include "rec/core_lt/Timer.h"

#include <boost/thread/mutex.hpp>

bool _run = true;

class MyLaserRangeFinder : public rec::robotino::com::LaserRangeFinder
{
public:
	MyLaserRangeFinder()
	{
	}

	virtual void readingsCallback( const rec::robotino::com::LaserRangeFinderReadings& readings )
	{
		float opening_angle = ( readings.angle_max - readings.angle_min );
		int mid_index = static_cast<int>( 0.5f * opening_angle / readings.angle_increment );

		std::cout << "Range | Intensity: " << readings.ranges[mid_index] << " | " << readings.intensities[mid_index] << std::endl;
	}

	virtual void error( const std::string& message )
	{
		_run = false;
		std::cout << "Error: " << message << std::endl;
	}

	virtual void connected()
	{
		std::cout << "connected" << std::endl;
	}

	virtual void disconnected()
	{
		_run = false;
		std::cout << "disconnected" << std::endl;
	}
};

MyLaserRangeFinder rangefinder;

void drive()
{
	while( _run )
	{
		rec::core_lt::msleep( 200 );
	}
}

int main( int argc, char **argv )
{
	std::string hostname = "127.0.0.1:8281";
	if( argc > 1 )
	{
		hostname = argv[1];
	}

	//this is only neccessary when not using rec::robotino::com::Com to connect to Robotino
	//When using Com the rangefinder connection is establish implicitly
	rangefinder.setAddress( hostname );
	rangefinder.connectToServer();

	drive();

	std::cout << "Press any key to exit..." << std::endl;
	rec::core_lt::waitForKey();
}
