//  Copyright (C) 2004-2010, Robotics Equipment Corporation GmbH

#define _USE_MATH_DEFINES
#include <cmath>
#include <iostream>

#include "rec/robotino/com/all.h"
#include "rec/core_lt/utils.h"
#include "rec/core_lt/Timer.h"

using namespace rec::robotino::com;

class MyCom : public Com
{
public:
	MyCom()
	{
	}

	void errorEvent( Error error, const char* errorString )
	{
		std::cerr << "Error: " << errorString << std::endl;
	}

	void connectedEvent()
	{
		std::cout << "Connected." << std::endl;
	}

	void connectionClosedEvent()
	{
		std::cout << "Connection closed." << std::endl;
	}

	void modeChangedEvent( bool isPassiveMode )
	{
		if( isPassiveMode )
		{
			std::cout << "Connected int passive mode." << std::endl;
		}
	}
};


MyCom com;
OmniDrive omniDrive;
Bumper bumper;
NorthStar nstar;

void init( const std::string& hostname )
{
	// Initialize the actors

	// Connect
	std::cout << "Connecting..." << std::endl;
	com.setAddress( hostname.c_str() );

	com.connect();

	std::cout << std::endl << "Connected" << std::endl;
}

void drive()
{
	nstar.setRoomId( 3 );
	nstar.setCeilingCal( 2800 );

	while( com.isConnected() && false == bumper.value() )
	{

		std::cout << "numspots: " << nstar.numSpotsVisible()
			<< "  spot0: " << nstar.magSpot0() << "  spot1: " << nstar.magSpot1()
			<< "  x,y,phi:" << nstar.posX() << "," << nstar.posY() << "," << nstar.posTheta()
			<< std::endl;;
		com.waitForUpdate();
	}
}

void destroy()
{
	com.disconnect();
}

int main( int argc, char **argv )
{
	std::string hostname = "172.26.1.1";
	if( argc > 1 )
	{
		hostname = argv[1];
	}

	try
	{
		init( hostname );
		drive();
		destroy();
	}
	catch( const rec::robotino::com::ComException& e )
	{
		std::cerr << "Com Error: " << e.what() << std::endl;
	}
	catch( const std::exception& e )
	{
		std::cerr << "Error: " << e.what() << std::endl;
	}
	catch( ... )
	{
		std::cerr << "Unknow Error" << std::endl;
	}

	//std::cout << "Press any key to exit..." << std::endl;
	//rec::core_lt::waitForKey();
}
