//  Copyright (C) 2004-2008, Robotics Equipment Corporation GmbH

#define _USE_MATH_DEFINES
#include <cmath>
#include <iostream>

#include "rec/robotino/com/all.h"
#include "rec/core_lt/utils.h"
#include "rec/core_lt/Timer.h"

using namespace rec::robotino::com;

class MyCom : public Com
{
public:
	MyCom()
	{
	}

	void errorEvent( Error error, const char* errorString )
	{
		std::cerr << "Error: " << errorString << std::endl;
	}

	void connectedEvent()
	{
		std::cout << "Connected." << std::endl;
	}

	void connectionClosedEvent()
	{
		std::cout << "Connection closed." << std::endl;
	}
};

MyCom com;
OmniDrive omniDrive;
Bumper bumper;

//rotate vector in by deg degrees and store the output in out
void rotate( const float* in, float* out, float deg )
{
	const float pi = 3.14159265358979f;

	float rad = 2 * pi / 360.0f * deg;

	out[0] = cos( rad ) * in[0] - sin( rad ) * in[1];
	out[1] = sin( rad ) * in[0] + cos( rad ) * in[1];
}

void init( const std::string& hostname )
{
	// Connect
	std::cout << "Connecting..." << std::endl;
	com.setAddress( hostname.c_str() );

	com.connect();

	std::cout << std::endl << "Connected" << std::endl;
}

void drive()
{
	rec::core_lt::Timer timer;
	timer.start();

	const float startVector[2] = {200.0f, 0.0f};
	float dir[2];
	float a = 0.0f;

	//omniDrive.setDriveLayout( 125.0, 35.0, 900.0, 16.0, 2000.0 );

	while( com.isConnected() && false == bumper.value() )
	{
		//rotate 360degrees in 10s
		rotate( startVector, dir, -1.0f * a );
		a = 360.0f * timer.msecsElapsed() / 10000;
		a = static_cast<int>(a) % 90;

		omniDrive.setVelocity( dir[0], dir[1], 36.0f );

		com.waitForUpdate();
	}
}

void destroy()
{
	com.disconnect();
}

int main( int argc, char **argv )
{
	std::string hostname = "172.26.1.1";
	if( argc > 1 )
	{
		hostname = argv[1];
	}

	try
	{
		init( hostname );
		drive();
		destroy();
	}
	catch( const rec::robotino::com::ComException& e )
	{
		std::cerr << "Com Error: " << e.what() << std::endl;
	}
	catch( const std::exception& e )
	{
		std::cerr << "Error: " << e.what() << std::endl;
	}
	catch( ... )
	{
		std::cerr << "Unknow Error" << std::endl;
	}

	//std::cout << "Press any key to exit..." << std::endl;
	//rec::core_lt::waitForKey();
}
