//  Copyright (C) 2004-2008, Robotics Equipment Corporation GmbH

#define _USE_MATH_DEFINES
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "rec/robotino/com/c/Com.h"

#ifdef WIN32
#include <windows.h>
// _getch
#include <conio.h>
#else
// getchar
#include <stdio.h>
// usleep
#include <unistd.h>
#endif

ComId com;
ManipulatorId manipulator;
BumperId bumper;

float cwMove[9] = { 0, 0, 0, 0, 0, 0, 0, 0, 0 };
float ccwMove[9] = { 0, 0, 0, 0, 0, 0, 0, 0, 0 };

void msleep( unsigned int ms )
{
#ifdef WIN32
	SleepEx( ms, FALSE );
#else
	usleep( ms * 1000 );
#endif
}

void waitForKey()
{
#ifdef WIN32
	_getch();
#else
	getchar();
#endif
}

void waitForAxesLimits()
{
	unsigned int seq;
	float angles[9];
	unsigned int numAngles;
	float speeds[9];
	unsigned int numSpeeds;
	int errors[9];
	unsigned int numErrors;
	int motors_enabled;
	int store_current_position;
	float cwAxesLimits[9];
	unsigned int numCwAxesLimits;
	float ccwAxesLimits[9];
	unsigned int numCcwAxesLimits;

	unsigned int i=0;

	while( Com_isConnected( com ) && FALSE == Bumper_value( bumper ) )
	{
		if( Manipulator_grab( manipulator ) )
		{
			Manipulator_getReadings(
				manipulator,
				&seq,
				angles,
				&numAngles,
				speeds,
				&numSpeeds,
				errors,
				&numErrors,
				&motors_enabled,
				&store_current_position,
				cwAxesLimits,
				&numCwAxesLimits,
				ccwAxesLimits,
				&numCcwAxesLimits );

			if( numCwAxesLimits > 0 )
			{
				for( i=0; i<numCwAxesLimits; ++i )
				{
					cwMove[i] = cwAxesLimits[i];
					if( cwMove[i] < 100 )
					{
						cwMove[i] = 100;
					}
				}

				for( i=0; i<numCcwAxesLimits; ++i )
				{
					ccwMove[i] = ccwAxesLimits[i];
					if( ccwMove[i] > 150 )
					{
						ccwMove[i] = 150;
					}
				}

				break;
			}
		}

		msleep( 500 );
	}
}

void move()
{
	float speedsSetPoint[9] = { 10, 10, 10, 10, 10, 10, 10, 10, 10 };

	unsigned int seq;
	float angles[9];
	unsigned int numAngles;
	float speeds[9];
	unsigned int numSpeeds;
	int errors[9];
	unsigned int numErrors;
	int motors_enabled;
	int store_current_position;
	float cwAxesLimits[9];
	unsigned int numCwAxesLimits;
	float ccwAxesLimits[9];
	unsigned int numCcwAxesLimits;

	int counter=0;
	unsigned int i;

	while( Com_isConnected( com ) && FALSE == Bumper_value( bumper ) )
	{
		if( Manipulator_grab( manipulator ) )
		{
			Manipulator_getReadings(
				manipulator,
				&seq,
				angles,
				&numAngles,
				speeds,
				&numSpeeds,
				errors,
				&numErrors,
				&motors_enabled,
				&store_current_position,
				cwAxesLimits,
				&numCwAxesLimits,
				ccwAxesLimits,
				&numCcwAxesLimits );

			printf( "angles: " );
			for( i=0; i<numAngles; ++i )
			{
				printf( "%f ", angles[i] );
			}
			printf( "\n" );
		}

		if( counter < 10 )
		{
			Manipulator_setAxes( manipulator, cwMove, 9, speedsSetPoint, 9 );
		}
		else if( counter < 20 )
		{
			Manipulator_setAxes( manipulator, ccwMove, 9, speedsSetPoint, 9 );
		}
		else
		{
			counter = 0;
		}

		msleep( 500 );
		++counter;
	}
}

void error( const char* message )
{
	printf( "%s\n", message );
	printf( "Press any key to exit..." );
	waitForKey();
	exit( 1 );
}

int main( int argc, char **argv )
{
	com = Com_construct();

	if( argc > 1 )
	{
		Com_setAddress( com, argv[1] );
	}
	else
	{
		//Com_setAddress( com, "172.26.1.1" );
		Com_setAddress( com, "127.0.0.1:8080" );
		//Com_setAddress( com, "192.168.3.2:8080" );
	}

	if( FALSE == Com_connect( com ) )
	{
		error( "Error on connect" );
	}
	else
	{
		char addressBuffer[256];
		Com_address( com, addressBuffer, 256 );
		printf( "Connected to %s\n", addressBuffer );
	}

	manipulator = Manipulator_construct();
	Manipulator_setComId( manipulator, com );

	bumper = Bumper_construct();
	Bumper_setComId( bumper, com );

	waitForAxesLimits();
	move();

	Manipulator_destroy( manipulator );
	Com_destroy( com );

	printf( "Press any key to exit...\n" );

	waitForKey();
}
