//  Copyright (C) 2004-2008, Robotics Equipment Corporation GmbH

#define _USE_MATH_DEFINES
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "rec/robotino/com/c/Com.h"

#ifdef WIN32
#include <windows.h>
// _getch
#include <conio.h>
#else
// getchar
#include <stdio.h>
// usleep
#include <unistd.h>
#endif

ComId com;
NorthStarId northstar;
BumperId bumper;

void msleep( unsigned int ms )
{
#ifdef WIN32
	SleepEx( ms, FALSE );
#else
	usleep( ms * 1000 );
#endif
}

void waitForKey()
{
#ifdef WIN32
	_getch();
#else
	getchar();
#endif
}

void drive()
{
	while( Com_isConnected( com ) && FALSE == Bumper_value( bumper ) )
  {
	  printf("SequenceNo:%d NumSpots:%d Spot0:%d Spot1:%d X:%d Y:%d Theta:%f \n"
		  , NorthStar_sequenceNo( northstar )
		  , NorthStar_numSpotsVisible( northstar )
		  , NorthStar_magSpot0( northstar ) 
		  , NorthStar_magSpot1( northstar )
		  , NorthStar_posX( northstar )
		  , NorthStar_posY( northstar )
		  , NorthStar_posTheta( northstar ) );

  }
}

void error( const char* message )
{
	printf( "%s\n", message );
	printf( "Press any key to exit..." );
    waitForKey();
	exit( 1 );
}

int main( int argc, char **argv )
{
	com = Com_construct();

	if( argc > 1 )
	{
		Com_setAddress( com, argv[1] );
	}
	else
	{
		Com_setAddress( com, "127.0.0.1:8080" );
		//Com_setAddress( com, "172.26.1.1" );
		//Com_setAddress( com, "192.168.101.101" );
	}

	if( FALSE == Com_connect( com ) )
	{
		error( "Error on connect" );
	}
	else
	{
		char addressBuffer[256];
		Com_address( com, addressBuffer, 256 );
		printf( "Connected to %s\n", addressBuffer );
	}

	northstar = NorthStar_construct();
	NorthStar_setComId( northstar, com );

	bumper = Bumper_construct();
	Bumper_setComId( bumper, com );

	drive();

	Bumper_destroy( bumper );
	NorthStar_destroy( northstar );
    Com_destroy( com );

	printf( "Press any key to exit...\n" );

    waitForKey();
}
