//  Copyright (C) 2004-2008, Robotics Equipment Corporation GmbH

#define _USE_MATH_DEFINES
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "rec/robotino/com/c/Com.h"

#ifdef WIN32
#include <windows.h>
// _getch
#include <conio.h>
#else
// getchar
#include <stdio.h>
// usleep
#include <unistd.h>
#endif

ComId com;
LaserRangeFinderId rangefinder;
BumperId bumper;

void msleep( unsigned int ms )
{
#ifdef WIN32
	SleepEx( ms, FALSE );
#else
	usleep( ms * 1000 );
#endif
}

void waitForKey()
{
#ifdef WIN32
	_getch();
#else
	getchar();
#endif
}

//rotate vector in by deg degrees and store the output in out
void rotate( const float* in, float* out, float deg )
{
	const float pi = 3.14159265358979f;

	float rad = 2 * pi / 360.0f * deg;

	out[0] = (float)( cos( rad ) * in[0] - sin( rad ) * in[1] );
	out[1] = (float)( sin( rad ) * in[0] + cos( rad ) * in[1] );
}

void grab()
{
	unsigned int seq;
	unsigned int stamp;
	float angle_min;
	float angle_max;
	float angle_increment;
	float time_increment;
	float scan_time;
	float range_min;
	float range_max;
	float ranges[1000];
	unsigned int numRanges;
	float intensities[1000];
	unsigned int numIntensities;

	float opening_angle;
	int mid_index;
	float mid;

	while( Com_isConnected( com ) && FALSE == Bumper_value( bumper ) )
	{
		if( LaserRangeFinder_grab( rangefinder ) )
		{
			LaserRangeFinder_getReadings(
				rangefinder,
				&seq,
				&stamp,
				&angle_min,
				&angle_max,
				&angle_increment,
				&time_increment,
				&scan_time,
				&range_min,
				&range_max,
				(float*)ranges,
				&numRanges,
				(float*)intensities,
				&numIntensities
				);

			opening_angle = ( angle_max - angle_min );
			mid_index = (int)( 0.5f * opening_angle / angle_increment );
			mid = ranges[mid_index];
			printf( "Received %u ranges and %u intensities. Mid range is %f meters \n", numRanges, numIntensities, mid );
		}

		msleep( 50 );
	}
}

void error( const char* message )
{
	printf( "%s\n", message );
	printf( "Press any key to exit..." );
	waitForKey();
	exit( 1 );
}

int main( int argc, char **argv )
{
	com = Com_construct();
	
	if( argc > 1 )
	{
		Com_setAddress( com, argv[1] );
	}
	else
	{
		//Com_setAddress( com, "172.26.1.1" );
		Com_setAddress( com, "127.0.0.1:8080" );
		//Com_setAddress( com, "192.168.3.2:8080" );
	}

	if( FALSE == Com_connect( com ) )
	{
		error( "Error on connect" );
	}
	else
	{
		char addressBuffer[256];
		Com_address( com, addressBuffer, 256 );
		printf( "Connected to %s\n", addressBuffer );
	}

	rangefinder = LaserRangeFinder_construct();
	LaserRangeFinder_setComId( rangefinder, com );

	bumper = Bumper_construct();
	Bumper_setComId( bumper, com );

	grab();

	LaserRangeFinder_destroy( rangefinder );
	Com_destroy( com );

	printf( "Press any key to exit...\n" );

	waitForKey();
}
