//  Copyright (C) 2004-2011, Robotics Equipment Corporation GmbH

#ifndef _REC_ROBOTINO_COM_COMPACTBHA_H_
#define _REC_ROBOTINO_COM_COMPACTBHA_H_

#include "rec/robotino/com/Actor.h"

#include <vector>
#include <string>

namespace rec
{
	namespace robotino
	{
		namespace com
		{
			class CompactBHAImpl;

			/**
			* @brief	Access to Robotino's (optional) compact version of the Bionic Handling Assistant. See <a href="http://www.festo.com/cms/en_corp/11367.htm">Robotino XT</a>.
			*/
			class
#ifdef WIN32
#  ifdef rec_robotino_com_EXPORTS
		__declspec(dllexport)
#  endif
#  ifdef rec_robotino_com2_EXPORTS
		__declspec(dllexport)
#  endif
#  ifdef rec_robotino_com3_EXPORTS
		__declspec(dllexport)
#  endif
#else
#  ifndef SWIG
    __attribute__ ((visibility ("default")))
#  endif
#endif
			CompactBHA : public Actor
			{
			public:
				CompactBHA();

				virtual ~CompactBHA();

				/** 
				* Sets the associated communication object.
				*
				* @param id	The id of the associated communication object.
				* @throws	RobotinoException If the given communication object doesn't provide a camera.
				*/
				void setComId( const ComId& id );

				/**
				Normally the CompactBHA's data exchange client is connected to Robotino by calling Com::connect().
				For debugging you might want to only connect to the CompactBHA server on Robotino.
				* @see Com::connect
				*/
				void connectToServer();

				/**
				Normally the CompactBHA's data exchange client is disconnected from Robotino by calling Com::disconnect().
				For debugging you might want to only disconnect from the CompactBHA server on Robotino.
				* @see Com::disconnect
				*/
				void disconnectFromServer();

				/**
				Normally the address of the CompactBHA server is given implicitly by calling Com::setAddress
				For debugging you might want to set the address explicitly.
				* @see Com::setAddress
				*/
				void setAddress( const std::string& address );

				std::string address() const;

				bool isConnected() const;

				void setPressures( const std::vector<float>& pressures );

				void setCompressorsEnabled( bool enabled );

				void setWaterDrainValve( bool enabled );

				void setGripperValve1( bool enabled );
				void setGripperValve2( bool enabled );

				virtual void readingsCallback( const std::vector<float>& pressures );

				virtual void pressureSensorCallback( bool pressureSensor );

				virtual void stringPotisCallback( const std::vector<float>& potis );

				virtual void foilPotiCallback( float poti );

				virtual void error( const std::string& message );

				virtual void log( const std::string& message );

				virtual void connected();

				virtual void disconnected();

			private:
				CompactBHAImpl* _impl;
			};
		}
	}
}

#endif //_REC_ROBOTINO_COM_COMPACTBHA_H_
