//  Copyright (C) 2004-2010, Robotics Equipment Corporation GmbH

#ifndef _REC_ROBOTINO_COM_LASERRANGEFINDER_H_
#define _REC_ROBOTINO_COM_LASERRANGEFINDER_H_

#include "rec/robotino/com/Actor.h"
#include "rec/robotino/com/LaserRangeFinderReadings.h"

#include <vector>
#include <string>

namespace rec
{
	namespace robotino
	{
		namespace com
		{
			class LaserRangeFinderImpl;

			/**
			* @brief	Access to Robotino's (optional) laser rangefinder
			*/
			class
#ifdef WIN32
#  ifdef rec_robotino_com_EXPORTS
		__declspec(dllexport)
#  endif
#  ifdef rec_robotino_com2_EXPORTS
		__declspec(dllexport)
#  endif
#  ifdef rec_robotino_com3_EXPORTS
		__declspec(dllexport)
#  endif
#else
#  ifndef SWIG
    __attribute__ ((visibility ("default")))
#  endif
#endif
			LaserRangeFinder : public Actor
			{
			public:
				LaserRangeFinder();

				virtual ~LaserRangeFinder();

				/** 
				* Sets the associated communication object.
				*
				* @param id	The id of the associated communication object.
				* @throws	RobotinoException If the given communication object doesn't provide a camera.
				*/
				void setComId( const ComId& id );

				/**
				Normally the LaserRangeFinder's data exchange client is connected to Robotino by calling Com::connect().
				For debugging you might want to only connect to the LaserRangeFinder server on Robotino.
				* @see Com::connect
				*/
				void connectToServer();

				/**
				Normally the LaserRangeFinder's data exchange client is disconnected from Robotino by calling Com::disconnect().
				For debugging you might want to only disconnect from the LaserRangeFinder server on Robotino.
				* @see Com::disconnect
				*/
				void disconnectFromServer();

				/**
				Normally the address of the LaserRangeFinder server is given implicitly by calling Com::setAddress
				For debugging you might want to set the address explicitly.
				* @see Com::setAddress
				*/
				void setAddress( const std::string& address );

				std::string address() const;

				bool isConnected() const;

				/**
				Enables/Disables the rawDataCallback and also disables/enables the readingsCallback.
				The rawDataCallback is disabled and the readingsCallback is enabled by default.
				@param enable True to enable the rawDataCallback. False to disable the rawDataCallback.
				*/
				void setRawDataCallbackEnabled( bool enable );

				/**
				@see setRawDataCallbackEnabled
				*/
				virtual void readingsCallback( const LaserRangeFinderReadings& scan );

				/**
				Use this callback to get the raw encoded rangefinder data. This data can be decoded by rec::core_lt::LaserScanMessage.
				@param data The encoded data
				@param dataSize The number of bytes in the data buffer.
				@see setRawDataCallbackEnabled
				*/
				virtual void rawDataCallback( const char* data, int dataSize );

				virtual void error( const std::string& message );

				virtual void log( const std::string& message );

				virtual void connected();

				virtual void disconnected();

			private:
				LaserRangeFinderImpl* _impl;
			};
		}
	}
}
#endif
