//  Copyright (C) 2004-2010, Robotics Equipment Corporation GmbH

#ifndef _REC_ROBOTINO_COM_MANIPULATORREADINGS_H_
#define _REC_ROBOTINO_COM_MANIPULATORREADINGS_H_

#include <vector>

namespace rec
{
	namespace robotino
	{
		namespace com
		{
			/**
			* @brief	Sensor readings of Robotino's (optional) manipulator
			*/
			class
#ifdef WIN32
#  ifdef rec_robotino_com_EXPORTS
		__declspec(dllexport)
#  endif
#  ifdef rec_robotino_com2_EXPORTS
		__declspec(dllexport)
#  endif
#  ifdef rec_robotino_com3_EXPORTS
		__declspec(dllexport)
#  endif
#else
#  ifndef SWIG
    __attribute__ ((visibility ("default")))
#  endif
#endif
			ManipulatorReadings
			{
			public:
				ManipulatorReadings()
					: motors_enabled( true )
					, store_current_position( false )
					, seq( 0 )
				{
				}

				/**
				Servo angles in deg
				*/
				std::vector<float> angles;

				/**
				Servo speeds in rpm
				*/
				std::vector<float> speeds;

				/**
				Servo errors.
				The error is a bit mask encoding the following errors
				Bit 0 - Input voltage error.
				Bit 1 - Angle limit error
				Bit 2 - Overheating error
				Bit 3 - range error
				Bit 4 - checksum error
				Bit 5 - Overload error
				Bit 6 - Instruction error
				Bit 7 - na
				*/
				std::vector<int> errors;

				bool motors_enabled;
				bool store_current_position;
			
				unsigned int seq;
			};
		}
	}
}

#endif //_REC_ROBOTINO_COM_MANIPULATORREADINGS_H_
